'use strict';
import { BufferLengthError } from './errors/BufferLengthError';
import { InvalidValueError } from './errors/InvalidValueError';
import { DPT } from './definitions';

const timeRegexp = /(\d{1,2}):(\d{1,2}):(\d{1,2})/;

export class DPT10 implements DPT {
  id = '10';
  name = 'day of week + time of day';
  bufferLength = 3;


  /**
   * Decode a buffer
   *
   * @param buffer the buffer
   * @returns the DPT value
   */
  decoder(buffer: Buffer): Date {
    if (buffer.length !== this.bufferLength)
      throw new BufferLengthError(`Invalid buffer length ${buffer.length}/${buffer}.  Expected ${this.bufferLength}.`);

    const [dnh, minutes, seconds] = buffer;
    const dow = (dnh & 0b11100000) >> 5;
    const hours = dnh & 0b00011111;

    if (hours < 0 || hours > 23)
      throw new InvalidValueError(`Hours out of range: Expected [0-23], got ${hours}`)
    if (minutes < 0 || minutes > 59)
      throw new InvalidValueError(`Minutes out of range: Expected [0-59], got ${minutes}`)
    if (seconds < 0 || seconds > 59)
      throw new InvalidValueError(`Seconds out of range: Expected [0-59], got ${seconds}`)

    const d = new Date();
    if (d.getDay() !== dow)
      // adjust day of month to get the day of week right
      d.setDate(d.getDate() + dow - d.getDay());
    // TODO: Shouldn't this be UTCHours?
    d.setHours(hours, minutes, seconds);
    // reset the milliseconds
    d.setMilliseconds(0)
    return d;
  };

  /**
   * Encode a buffer
   *
   * @param value the value to be converted to buffer
   * @returns the buffer
   */
  encoder(value: Date | number | string): Buffer {
    if (value === undefined || value === null)
      throw new InvalidValueError(`Must supply a Date, number or String for DPT10 time.`);

    let dow, hour, minute, second;
    // day of week. NOTE: JS Sunday = 0
    switch (typeof value) {
      case 'string':
        // try to parse
        let match = timeRegexp.exec(value);
        if (match) {
          dow = ((new Date().getDay() - 7) % 7) + 7;
          hour = parseInt(match[1]);
          minute = parseInt(match[2]);
          second = parseInt(match[3]);
        } else {
          throw new InvalidValueError(`DPT10: invalid time format (${value})`);
        }
        break;
      case 'number':
        value = new Date(value);
      default:
        dow = ((value.getDay() - 7) % 7) + 7;
        hour = value.getHours();
        minute = value.getMinutes();
        second = value.getSeconds();
    }

    return Buffer.from([(dow << 5) + hour, minute, second]);
  }


  subtypes: {
    // 10.001 time of day
    '001': {
      name: 'DPT_TimeOfDay',
      desc: 'time of day',
    },
  };

}
