'use strict';
import { BufferLengthError } from './errors/BufferLengthError';
import { InvalidValueError } from './errors/InvalidValueError';
import { DPT } from './definitions';

/**
 * <strong>11.001</strong> Date
 *
 * <pre>
 *             +-7-+-6-+-5-+-4-+-3-+-2-+-1-+-0-+
 * Field Names | 0   0   0   (Day)             |
 * Encoding    |             U   U   U   U   U |
 *             +-7-+-6-+-5-+-4-+-3-+-2-+-1-+-0-+
 *             | 0   0   0   0   (Month)       |
 *             |                 U   U   U   U |
 *             +-7-+-6-+-5-+-4-+-3-+-2-+-1-+-0-+
 *             | 0   (Year)                    |
 *             |     U   U   U   U   U   U   U |
 *             +---+---+---+---+---+---+---+---+
 * Format:     3 octets (r<sub>3</sub> U<sub>5</sub> r<sub>4</sub> U<sub>4</sub> r<sub>1</sub> U<sub>7</sub>)
 * Encoding:   Day   = [1 .. 31]
 *             Month = [1 .. 12]
 *             Year  = [0 .. 99]
 * </pre>
 * <p>
 * This format covers the range 1990 to 2089. The following interpretation shall be carried out by devices receiving
 * the Data Point Type 11.001 and carrying out calculations on the basis of the entire 3rd octet:
 * <p>
 * - If Octet 3 contains value ≥ 90 : interpret as 20th century<br>
 * - If Octet 3 contains value < 90: interpret as 21st century<br>
 */

export class DPT11 implements DPT {
  id = '11';
  name = '3-byte date value';
  bufferLength = 3;

  /**
   * Decode a buffer
   *
   * @param buffer the buffer
   * @returns the DPT value
   */
  decoder(buffer: Buffer): Date {
    if (buffer.length !== this.bufferLength)
      throw new BufferLengthError(`Invalid buffer length ${buffer.length}/${buffer}.  Expected ${this.bufferLength}.`);

    const day = buffer[0] & 31; //0b00011111);
    const month = buffer[1] & 15; //0b00001111);
    let year = buffer[2] & 127; //0b01111111);
    year = (year < 90 ? 2000 : 1900) + year
    if (
      day < 1 ||
      day > 31 ||
      month < 1 ||
      month > 12 ||
      year < 1990 ||
      year > 2089
    ) {
      throw new InvalidValueError(`${buffer} => ${day}/${month}/${year} is not valid date according to DPT11, setting to 1990/01/01`);
    }
    return new Date(year, month - 1, day);
  };

  /**
   * Encode a buffer
   *
   * @param value the value to be converted to buffer
   * @returns the buffer
   */
  encoder(value: Date | string): Buffer {
    if (value === undefined || value === null)
      throw new InvalidValueError('Cannot write null value');

    switch (typeof value) {
      case 'string':
      case 'number':
        value = new Date(value);
        break;
    }

    if (isNaN(value.getDate()))
      throw new InvalidValueError('Must supply a numeric timestamp, Date or String object for DPT11 Date')

    const year = value.getFullYear();
    return Buffer.from([
      value.getDate(),
      value.getMonth() + 1,
      (year % 100),
    ]);
  }


  subtypes: {
    // 11.001 date
    '001': {
      name: 'DPT_Date',
      desc: 'Date',
    },
  };

}
