'use strict';
import { BufferLengthError } from './errors/BufferLengthError';
import { InvalidValueError } from './errors/InvalidValueError';
import { DPT } from './definitions';

export class DPT14 implements DPT {
  id = '14';
  name = '32-bit floating point value';
  bufferLength = 4;


  /**
   * Decode a buffer
   *
   * @param buffer the buffer
   * @returns the DPT value
   */
  decoder(buffer: Buffer): number {
    if (buffer.length !== this.bufferLength)
      throw new BufferLengthError(`Invalid buffer length ${buffer.length}/${buffer}.  Expected ${this.bufferLength}.`);

    return buffer.readFloatBE(0);
  };

  /**
   * Encode a buffer
   *
   * @param value the value to be converted to buffer
   * @returns the buffer
   */
  encoder(value: number): Buffer {
    if (!value || typeof value != 'number')
      throw new InvalidValueError(`Invalid value [${value}]`)
    const apdu_data = Buffer.alloc(this.bufferLength);
    apdu_data.writeFloatBE(value, 0);
    return apdu_data;
  }


  subtypes: {
    // TODO
    '007': {
      name: 'DPT_Value_AngleDeg°',
      desc: 'angle, degree',
      unit: '°',
    },

    '019': {
      name: 'DPT_Value_Electric_Current',
      desc: 'electric current',
      unit: 'A',
    },

    '027': {
      name: 'DPT_Value_Electric_Potential',
      desc: 'electric potential',
      unit: 'V',
    },

    '028': {
      name: 'DPT_Value_Electric_PotentialDifference',
      desc: 'electric potential difference',
      unit: 'V',
    },

    '031': {
      name: 'DPT_Value_Energ',
      desc: 'energy',
      unit: 'J',
    },

    '032': {
      name: 'DPT_Value_Force',
      desc: 'force',
      unit: 'N',
    },

    '033': {
      name: 'DPT_Value_Frequency',
      desc: 'frequency',
      unit: 'Hz',
    },

    '036': {
      name: 'DPT_Value_Heat_FlowRate',
      desc: 'heat flow rate',
      unit: 'W',
    },

    '037': {
      name: 'DPT_Value_Heat_Quantity',
      desc: 'heat, quantity of',
      unit: 'J',
    },

    '038': {
      name: 'DPT_Value_Impedance',
      desc: 'impedance',
      unit: 'Ω',
    },

    '039': {
      name: 'DPT_Value_Length',
      desc: 'length',
      unit: 'm',
    },

    '051': {
      name: 'DPT_Value_Mass',
      desc: 'mass',
      unit: 'kg',
    },

    '056': {
      name: 'DPT_Value_Power',
      desc: 'power',
      unit: 'W',
    },

    '065': {
      name: 'DPT_Value_Speed',
      desc: 'speed',
      unit: 'm/s',
    },

    '066': {
      name: 'DPT_Value_Stress',
      desc: 'stress',
      unit: 'Pa',
    },

    '067': {
      name: 'DPT_Value_Surface_Tension',
      desc: 'surface tension',
      unit: '1/Nm',
    },

    '068': {
      name: 'DPT_Value_Common_Temperature',
      desc: 'temperature, common',
      unit: '°C',
    },

    '069': {
      name: 'DPT_Value_Absolute_Temperature',
      desc: 'temperature (absolute)',
      unit: 'K',
    },

    '070': {
      name: 'DPT_Value_TemperatureDifference',
      desc: 'temperature difference',
      unit: 'K',
    },

    '078': {
      name: 'DPT_Value_Weight',
      desc: 'weight',
      unit: 'N',
    },

    '079': {
      name: 'DPT_Value_Work',
      desc: 'work',
      unit: 'J',
    },
  };

}
