'use strict';
import { BufferLengthError } from './errors/BufferLengthError';
import { InvalidValueError } from './errors/InvalidValueError';
import { DPT } from './definitions';

export class DPT17 implements DPT {
  id = '17';
  name = 'scene number';
  bufferLength = 1;

  /**
   * Decode a buffer
   *
   * @param buffer the buffer
   * @returns the DPT value
   */
  decoder(buffer: Buffer): number {
    if (buffer.length !== this.bufferLength)
      throw new BufferLengthError(`Invalid buffer length ${buffer.length}/${buffer}.  Expected ${this.bufferLength}.`);

    // Scene number between 0 and 63
    return buffer.readUInt8(0) & 0b00111111;
  };

  /**
   * Encode a buffer
   *
   * @param value the value to be converted to buffer
   * @returns the buffer
   */
  encoder(value: number): Buffer {
    if (value === undefined || value === null)
      throw new InvalidValueError(`Invalid value [${value}]`)

    if (value > 63)
      throw new InvalidValueError(`Expected scene number [0, 63]. Got [${value}]`)

    let buf = Buffer.alloc(this.bufferLength)
    buf.writeUInt8(value, 0)
    return buf;
  }


  subtypes: {
    // 17.001 Scene number
    "001": {
      use: "G",
      name: "DPT_SceneNumber", desc: "Scene Number",
    },
  };

}
