'use strict';
import { BufferLengthError } from './errors/BufferLengthError';
import { InvalidValueError } from './errors/InvalidValueError';
import { DPT } from './definitions';

export interface DPT18Result {
  activateLearn: number
  // pad: number // reserved
  sceneNumber: number
}
export class DPT18 implements DPT {
  id = '18';
  name = '8-bit Scene Activate/Learn + number';
  bufferLength = 1;

  /**
   * Decode a buffer
   *
   * @param buffer the buffer
   * @returns the DPT value
   */
  decoder(buffer: Buffer): DPT18Result {
    if (buffer.length !== this.bufferLength)
      throw new BufferLengthError(`Invalid buffer length ${buffer.length}/${buffer}.  Expected ${this.bufferLength}.`);


    return {
      activateLearn: (buffer[0] & 0b10000000) >> 7,
      sceneNumber: buffer[0] & 0b00111111
    };
  };

  /**
   * Encode a buffer
   *
   * @param value the value to be converted to buffer
   * @returns the buffer
   */
  encoder(value: DPT18Result): Buffer {
    if (value === undefined || value === null)
      throw new InvalidValueError(`Invalid value [${value}]`)

    if (value.sceneNumber > 63)
      throw new InvalidValueError(`Expected scene number [0..63]. Got [${value.sceneNumber}]`)

    if (value.activateLearn > 1)
      throw new InvalidValueError(`Expected scene number [0, 1]. Got [${value.activateLearn}]`)

    let buf = Buffer.alloc(this.bufferLength)
    buf.writeUInt8((value.activateLearn << 7) + value.sceneNumber)
    return buf;
  }


  subtypes: {
    // 9.001 temperature (oC)
    "001": {
      name: "DPT_SceneControl", desc: "scene control"
    }
  };

}
