'use strict';
import { BufferLengthError } from './errors/BufferLengthError';
import { InvalidValueError } from './errors/InvalidValueError';
import { DPT } from './definitions';

/**
 * Original at https://github.com/pitschr/knx-core/blob/main/src/main/java/li/pitschmann/knx/core/datapoint/DPT20.java
 *
 * Data Point Type 20 for '8-Bit Enumeration' (1 Octet)
 *
 * <pre>
 *             +-7-+-6-+-5-+-4-+-3-+-2-+-1-+-0-+
 * Field Names | (Field 1)                     |
 * Encoding    | N   N   N   N   N   N   N   N |
 *             +---+---+---+---+---+---+---+---+
 * Format:     8 bit (N<sub>8</sub>)
 * </pre>
 */
export class DPT20 implements DPT {
  id = '20';
  name = '8-Bit Enumeration';
  bufferLength = 1;

  /**
   * Decode a buffer
   *
   * @param buffer the buffer
   * @returns the DPT value
   */
  decoder(buffer: Buffer): number {
    if (buffer.length !== this.bufferLength)
      throw new BufferLengthError(`Invalid buffer length ${buffer.length}/${buffer}.  Expected ${this.bufferLength}.`);

    return buffer.readUInt8(0);
  };

  /**
   * Encode a buffer
   *
   * @param value the value to be converted to buffer
   * @returns the buffer
   */
  encoder(value: number): Buffer {
    if (value === undefined || value === null)
      throw new InvalidValueError(`Invalid value [${value}]`)

    return Buffer.from([value & 0xff])
  }


  subtypes: {
    // 20.001 SCLO Mode
    1: {
      name: 'SCLO_Mode',
      desc: '',
      unit: '',
      scalar_range: {
        0: 'Autonomous',
        1: 'Slave',
        2: 'Master'
      },
      range: undefined,
    },

    // 20.001 Building Mode
    2: {
      name: 'Building_Mode',
      desc: '',
      unit: '',
      scalar_range: {
        0: 'Building in use',
        1: 'Building not used',
        2: 'Building protection'
      },
      range: undefined,
    },

    // 20.102 HVAC mode
    102: {
      name: 'HVAC_Mode',
      desc: '',
      unit: '',
      scalar_range: {
        0: 'Auto',
        1: 'Comfort',
        2: 'Standby',
        3: 'Economy',
        4: 'Building protection'
      },
      range: undefined,
    },
  };

}
