'use strict';
import { BufferLengthError } from './errors/BufferLengthError';
import { InvalidValueError } from './errors/InvalidValueError';
import { DPT } from './definitions';

/**
 * Original at https://github.com/pitschr/knx-core/blob/main/src/main/java/li/pitschmann/knx/core/datapoint/DPT21.java
 * * Data Point Type 21 for 8-Bits flagged messages
 *
 * <pre>
 *             +-7-+-6-+-5-+-4-+-3-+-2-+-1-+-0-+
 * Field Names | b   b   b   b   b   b   b   b |
 * Encoding    | B   B   B   B   B   B   B   B |
 *             +---+---+---+---+---+---+---+---+
 * Format:     8 bits (B<sub>8</sub>)
 * </pre>
 */
export class DPT21 implements DPT {
  id = '';
  name = '';
  bufferLength = 1;

  /**
   * Decode a buffer
   *
   * @param buffer the buffer
   * @returns the DPT value
   */
  decoder(buffer: Buffer): boolean[] {
    if (buffer.length !== this.bufferLength)
      throw new BufferLengthError(`Invalid buffer length ${buffer.length}/${buffer}.  Expected ${this.bufferLength}.`);

    const result = [
      !!(buffer[0] & 0x80),
      !!(buffer[0] & 0x40),
      !!(buffer[0] & 0x20),
      !!(buffer[0] & 0x10),
      !!(buffer[0] & 0x08),
      !!(buffer[0] & 0x04),
      !!(buffer[0] & 0x02),
      !!(buffer[0] & 0x01),
    ]
    return result;
  };

  /**
   * Encode a buffer
   *
   * @param value the value to be converted to buffer
   * @returns the buffer
   */
  encoder(value: boolean[]): Buffer {
    if (value === undefined || value === null)
      throw new InvalidValueError(`Invalid value [${value}]`)

    if (value.length != 8)
      throw new InvalidValueError(`Value length should be 8. Got ${value.length}.`)

    const b7 = value[0] ? 1 : 0
    const b6 = value[1] ? 1 : 0
    const b5 = value[2] ? 1 : 0
    const b4 = value[3] ? 1 : 0
    const b3 = value[4] ? 1 : 0
    const b2 = value[5] ? 1 : 0
    const b1 = value[6] ? 1 : 0
    const b0 = value[7] ? 1 : 0

    const buf = Buffer.from([
      b7 << 7 |
      b6 << 6 |
      b5 << 5 |
      b4 << 4 |
      b3 << 3 |
      b2 << 2 |
      b1 << 1 |
      b0
    ])
    return buf;
  }


  subtypes: {
    // 21.001 status - 5 bits
    "001": {
      "name": "DPT_StatusGen",
      "desc": "General Status",
      "unit": "",
      "scalar_range": undefined,
      "range": undefined
    },
    // 21.002 control - 3 bits
    "002": {
      "name": "DPT_Device_Control",
      "desc": "Device Control",
      "unit": "",
      "scalar_range": undefined,
      "range": undefined
    }
  };

}
