'use strict';
import { BufferLengthError } from './errors/BufferLengthError';
import { InvalidValueError } from './errors/InvalidValueError';
import { DPT } from './definitions';
export interface DPT232Result {
  red: number,
  green: number,
  blue: number
}
export class DPT232 implements DPT {
  id = '232';
  name = 'RGB array';
  bufferLength = 3

  /**
   * Decode a buffer
   *
   * @param buffer the buffer
   * @returns the on/off value
   */
  decoder(buffer: Buffer): DPT232Result {
    if (buffer.length !== this.bufferLength)
      throw new BufferLengthError(`Invalid buffer length ${buffer.length}/${buffer} for DPT1.  Expected ${this.bufferLength}.`);

    const result: DPT232Result = {
      red: buffer.readUInt8(0),
      green: buffer.readUInt8(1),
      blue: buffer.readUInt8(2),
    }

    return result;
  };

  /**
   * Encode a buffer
   *
   * @param value the value to be converted to buffer
   * @returns the buffer
   */
  encoder(value: DPT232Result): Buffer {
    if (value === undefined || value === null)
      throw new InvalidValueError('Cannot write null value');

    if (value.red < 0 || value.red > 255)
      throw new InvalidValueError(`Red component out of range ${value.red}`);
    if (value.blue < 0 || value.blue > 255)
      throw new InvalidValueError(`Blue component out of range ${value.blue}`);
    if (value.green < 0 || value.green > 255)
      throw new InvalidValueError(`Green component out of range ${value.green}`);

    let buffer = Buffer.alloc(this.bufferLength)
    buffer.writeUInt8(value.red, 0)
    buffer.writeUInt8(value.green, 1)
    buffer.writeUInt8(value.blue, 2)

    return buffer
  }

  subtypes: {
    '600': {
      name: 'RGB',
      desc: 'RGB color triplet',
      unit: '',
    },
  };
}
