'use strict';
import { BufferLengthError } from './errors/BufferLengthError';
import { InvalidValueError } from './errors/InvalidValueError';
import { DPT } from './definitions';
import { dec2bin, hex2bin } from './convertors';

/**
 * This is a class with flags!
 */

export interface DPT237Result {
  readResponse: boolean
  addressIndicator: boolean
  daliAddress: number

  lampFailure: boolean
  ballastFailure: boolean
  convertorError: boolean
}
export class DPT237 implements DPT {
  id = '237';
  name = '2-byte unsigned value';
  bufferLength = 2;


  /**
   * Decode a buffer
   *
   * @param buffer the buffer
   * @returns the DPT value
   */
  decoder(buffer: Buffer): DPT237Result {
    if (buffer.length !== this.bufferLength)
      throw new BufferLengthError(`Invalid buffer length ${buffer.length}/${buffer}.  Expected ${this.bufferLength}.`);

    let str = buffer.toString('hex');

    let MSB = hex2bin(str.substring(0, 2)); // Get Binary
    let LSB = hex2bin(str.substring(2, 4));// Get Binary

    return {
      // LSB
      readResponse: LSB.substring(0, 1) === "0" ? false : true,
      addressIndicator: LSB.substring(1, 2) === "0" ? false : true,
      daliAddress: parseInt("00" + LSB.substring(2, 8), 2),

      // MSB
      lampFailure: MSB.substring(7, 8) === "0" ? false : true,
      ballastFailure: MSB.substring(6, 7) === "0" ? false : true,
      convertorError: MSB.substring(5, 6) === "0" ? false : true,
    }
  };

  /**
   * Encode a buffer
   *
   * @param value the value to be converted to buffer
   * @returns the buffer
   */
  encoder(value: DPT237Result): Buffer {
    if (value === undefined || value === null)
      throw new InvalidValueError('Cannot write null value');

    if (value.daliAddress < 0 || value.daliAddress > 63)
      throw new InvalidValueError(`daliAddress must be between [0, 63]. git ${value.daliAddress}`);


    // LSB
    let LSB = (value.readResponse === false ? "0" : "1") +
      (value.addressIndicator === false ? "0" : "1") +
      dec2bin(value.daliAddress).padStart(6, "0");

    // MSB
    let MSB = "00000" + (value.convertorError === false ? "0" : "1") +
      (value.ballastFailure === false ? "0" : "1") +
      (value.lampFailure === false ? "0" : "1");

    var bufferTotal = Buffer.alloc(2)
    bufferTotal[0] = parseInt(MSB, 2);
    bufferTotal[1] = parseInt(LSB, 2);
    return bufferTotal;
  }


  subtypes: {
    "600": {
      "desc": "DPT_DALI_Control_Gear_Diagnostic",
      "name": "DALI control gear diagnostic",
      "unit": "",
      //"scalar_range": [,],
      //"range": [,]
    }
  };

}
