'use strict';
import { BufferLengthError } from './errors/BufferLengthError';
import { InvalidValueError } from './errors/InvalidValueError';
import { DPT } from './definitions';

export class DPT238 implements DPT {
  id = '238';
  name = '8-bit unsigned value';
  bufferLength = 1

  decoder(buffer: Buffer): number {
    if (buffer.length !== this.bufferLength)
      throw new BufferLengthError(`Invalid buffer length ${buffer.length}/${buffer} for DPT1.  Expected ${this.bufferLength}.`);

    return buffer.readUInt8(0)
  }

  encoder(value: number): Buffer {
    if (!value)
      throw new InvalidValueError('DPT5: cannot write null value');

    let buf = Buffer.alloc(this.bufferLength);
    buf.writeUInt8(value, 0)
    return buf;

  }

  subtypes: {
    // 20.102 HVAC mode
    102: {
      name: 'HVAC_Mode',
      desc: '',
      unit: '',
      //scalar_range: [,],
      //range: [,],
    },

    // 5.003 angle (degrees 0=0, ff=360)
    '003': {
      name: 'DPT_Angle',
      desc: 'angle degrees',
      unit: '°',
      scalar_range: [0, 360],
    },

    // 5.004 percentage (0..255%)
    '004': {
      name: 'DPT_Percent_U8',
      desc: 'percent',
      unit: '%',
    },

    // 5.005 ratio (0..255)
    '005': {
      name: 'DPT_DecimalFactor',
      desc: 'ratio',
      unit: 'ratio',
    },

    // 5.006 tariff (0..255)
    '006': {
      name: 'DPT_Tariff',
      desc: 'tariff',
      unit: 'tariff',
    },

    // 5.010 counter pulses (0..255)
    '010': {
      name: 'DPT_Value_1_Ucount',
      desc: 'counter pulses',
      unit: 'pulses',
    },
  }
}