'use strict';
import { BufferLengthError } from './errors/BufferLengthError';
import { InvalidValueError } from './errors/InvalidValueError';
import { DPT } from './definitions';

export interface DPT3Result {
  decr_incr: number;
  data: number;
}

export class DPT3 implements DPT {
  id = '3';
  name = '4-bit relative dimming control';
  bufferLength = 1;

  /**
   * Decode a buffer
   *
   * @param buffer the buffer
   * @returns the DPT value
   */
  decoder(buffer: Buffer): DPT3Result {
    if (buffer.length !== this.bufferLength)
      throw new BufferLengthError(`Invalid buffer length ${buffer.length}/${buffer} for DPT1.  Expected ${this.bufferLength}.`);

    const value = buffer.readUInt8(0)
    return {
      decr_incr: (value & 0b00001000) >> 3,
      data: value & 0b00000111,
    };
  };

  /**
   * Encode a buffer
   *
   * @param value the value to be converted to buffer
   * @returns the buffer
   */
  encoder(value: DPT3Result): Buffer {
    if (!value)
      throw new InvalidValueError('DPT3: cannot write null value');

    if (value.decr_incr > 1)
      throw new InvalidValueError(`DPT3: Invalid increment/decrement value: ${value.decr_incr}. Expected 1 or 0.`);

    return Buffer.from([(value.decr_incr << 3) + (value.data & 0b00000111)]);

  }

  subtypes: {
    // 3.007 dimming control
    '007': {
      name: 'DPT_Control_Dimming',
      desc: 'dimming control',
    },

    // 3.008 blind control
    '008': {
      name: 'DPT_Control_Blinds',
      desc: 'blinds control',
    },
  };

}
