'use strict';
import { BufferLengthError } from './errors/BufferLengthError';
import { InvalidValueError } from './errors/InvalidValueError';
import { DPT } from './definitions';

export class DPT4 implements DPT {
  id = '4';
  name = '8-bit character';
  bufferLength = 1;

  /**
   * Decode a buffer
   *
   * @param buffer the buffer
   * @returns the DPT value
   */
  decoder(buffer: Buffer): string {
    if (buffer.length !== this.bufferLength)
      throw new BufferLengthError(`Invalid buffer length ${buffer.length}/${buffer} for DPT1.  Expected ${this.bufferLength}.`);

    return String.fromCharCode(buffer[0]);
  };

  /**
   * Encode a buffer
   *
   * @param value the value to be converted to buffer
   * @returns the buffer
   */
  encoder(value: string): Buffer {
    if (!value)
      throw new InvalidValueError('DPT4: cannot write null value');

    const apdu_data = value.charCodeAt(0);
    if (apdu_data > 255)
      throw new InvalidValueError('DPT4: must supply an ASCII character');

    return Buffer.from([apdu_data]);
  }

  subtypes: {
    // 4.001 character (ASCII)
    '001': {
      name: 'DPT_Char_ASCII',
      desc: 'ASCII character (0-127)',
      range: [0, 127],
      use: 'G',
    },
    // 4.002 character (ISO-8859-1)
    '002': {
      name: 'DPT_Char_8859_1',
      desc: 'ISO-8859-1 character (0..255)',
      use: 'G',
    },
  };

}
