'use strict';
import { BufferLengthError } from './errors/BufferLengthError';
import { InvalidValueError } from './errors/InvalidValueError';
import { DPT } from './definitions';

export class DPT6 implements DPT {
  id = '6';
  name = "8-bit signed value";
  bufferLength = 1;
  range = [-128, 127];

  /**
   * Decode a buffer
   *
   * @param buffer the buffer
   * @returns the DPT value
   */
  decoder(buffer: Buffer): number {
    if (buffer.length !== this.bufferLength)
      throw new BufferLengthError(`Invalid buffer length ${buffer.length}/${buffer} for DPT1.  Expected ${this.bufferLength}.`);

    const value = buffer.readInt8(0)

    return value;
  };

  /**
   * Encode a buffer
   *
   * @param value the value to be converted to buffer
   * @returns the buffer
   */
  encoder(value: number): Buffer {
    if (!value)
      throw new InvalidValueError('DPT6: cannot write null value');

    if (value < this.range[0] || value > this.range[1])
      throw new InvalidValueError(`DPT6: Value ${value} out of range [${this.range[0]}, ${this.range[1]}].`);

    let buf = Buffer.alloc(this.bufferLength);
    buf.writeInt8(value, 0)
    return buf;
  }

  subtypes: {
    // 6.001 percentage (-128%..127%)
    "001": {
      "name": "DPT_Switch", "desc": "percent",
      "unit": "%",
    },

    // 6.002 counter pulses (-128..127)
    "010": {
      "name": "DPT_Bool", "desc": "counter pulses",
      "unit": "pulses"
    },
  };
}
