'use strict';
import { BufferLengthError } from './errors/BufferLengthError';
import { InvalidValueError } from './errors/InvalidValueError';
import { DPT } from './definitions';

export class DPT8 implements DPT {
  id = '8';
  name = '16-bit signed value';
  bufferLength = 2;
  "range" = [-32768, 32767];

  /**
   * Decode a buffer
   *
   * @param buffer the buffer
   * @returns the DPT value
   */
  decoder(buffer: Buffer): number {
    if (buffer.length !== this.bufferLength)
      throw new BufferLengthError(`Invalid buffer length ${buffer.length}/${buffer} for DPT1.  Expected ${this.bufferLength}.`);

    const value = buffer.readInt16BE(0)

    return value;
  };

  /**
   * Encode a buffer
   *
   * @param value the value to be converted to buffer
   * @returns the buffer
   */
  encoder(value: number): Buffer {
    if (!value)
      throw new InvalidValueError('DPT8: cannot write null value');

    if (value < this.range[0] || value > this.range[1])
      throw new InvalidValueError(`DPT6: Value ${value} out of range [${this.range[0]}, ${this.range[1]}].`);

    let buf = Buffer.alloc(this.bufferLength);
    buf.writeInt16BE(value, 0)
    return buf;
  }


  subtypes: {
    // 8.001 pulses difference
    "001": {
      "name": "DPT_Value_2_Count",
      "desc": "pulses",
      "unit": "pulses"
    },

    // 8.002 time lag (ms)
    "002": {
      "name": "DPT_DeltaTimeMsec",
      "desc": "time lag(ms)",
      "unit": "milliseconds"
    },

    // 8.003 time lag (10ms)
    "003": {
      "name": "DPT_DeltaTime10Msec",
      "desc": "time lag(10ms)",
      "unit": "centiseconds"
    },

    // 8.004 time lag (100ms)
    "004": {
      "name": "DPT_DeltaTime100Msec",
      "desc": "time lag(100ms)",
      "unit": "deciseconds"
    },

    // 8.005 time lag (sec)
    "005": {
      "name": "DPT_DeltaTimeSec",
      "desc": "time lag(s)",
      "unit": "seconds"
    },

    // 8.006 time lag (min)
    "006": {
      "name": "DPT_DeltaTimeMin",
      "desc": "time lag(min)",
      "unit": "minutes"
    },

    // 8.007 time lag (hour)
    "007": {
      "name": "DPT_DeltaTimeHrs",
      "desc": "time lag(hrs)",
      "unit": "hours"
    },

    // 8.010 percentage difference (%)
    "010": {
      "name": "DPT_Percent_V16",
      "desc": "percentage difference",
      "unit": "%"
    },

    // 8.011 rotation angle (deg)
    "011": {
      "name": "DPT_RotationAngle",
      "desc": "angle(degrees)",
      "unit": "°"
    },
  };

}
